#!/usr/bin/env python
#
# This script writes the cpp fixed order
# splitting code using the generating
# functional obtained with configs.pl
#

import cmdline

def main():
    order=cmdline.value('-order',1)
    cmdline.assert_all_options_used()
    ind='  '
    cpp=open('FixedOrderGenerator.cc','w')
    cpp.write('/// \\file FixedOrderGenerator.cc This file is generated automatically by writecpp.py.\\n\n')
    cpp.write('/// It contains the code for fixed order n<='+str(order)+
              ' splitting of a quark or gluon.\\n\n')
    cpp.write('/// For initial quark use split_quarkN() for Nth order splitting.\\n\n')
    cpp.write('/// For initial gluon use split_gluonN() for Nth order splitting.\\n\n')
    cpp.write('#include "FixedOrderGenerator.hh"\n\n')
    cpp.write('const double FixedOrderGenerator::_eps = 1e-10;\n\n')
    for k in xrange(order):
        filename_quark = 'functional/quark_order'+str(k+1)+'.dat'
        cpp.write('void FixedOrderGenerator::split_quark'+str(k+1)+'() {\n')
        write(filename_quark,cpp,k+1,True)
        cpp.write('}\n')
        filename_gluon = 'functional/gluon_order'+str(k+1)+'.dat'
        cpp.write('void FixedOrderGenerator::split_gluon'+str(k+1)+'() {\n')
        write(filename_gluon,cpp,k+1,False)
        cpp.write('}\n')
    for k in xrange(order,5):
        cpp.write('void FixedOrderGenerator::split_quark'+str(k+1)+'() {\n')
        cpp.write(ind+'std::cout << "Invalid value for order" << std::endl;\n')
        cpp.write(ind+'exit(-1);\n')
        cpp.write('}\n')
        cpp.write('void FixedOrderGenerator::split_gluon'+str(k+1)+'() {\n')
        cpp.write(ind+'std::cout << "Invalid value for order" << std::endl;\n')
        cpp.write(ind+'exit(-1);\n')
        cpp.write('}\n')

def write(filename,cpp,order,quark):
    ind='  '
    cpp.write(ind+'/// Code generated automatically by writecpp.py using ' + filename + '.\n')
    n=0
    d={}
    for line in open(filename,'r'):
        if line[0] != '#' and line.strip():
            str_prod = ''
            for token in line.split():
                if token[0] == 'p':
                    str_prod+=str(token)+'*'
            str_prod=str_prod[:-1]
            if not d.get(str_prod):
                d[str_prod]='weight['+str(n)+']'
                n+=1
    cpp.write(ind+'double weight['+str(n)+'];\n')
    cpp.write(ind+'double z=_rng.uniform();\n')
    if quark:
        cpp.write(ind+'double pqq_z=pqq(z);\n')
    else:
        cpp.write(ind+'double pqg_z=pqg(z), ')
        cpp.write('pgg_z=pgg(z);\n')
    for i in xrange(1,order):
        cpp.write(ind+'double z'+str('p'*i)+'=_rng.uniform();\n')
        cpp.write(ind+'double pqq_z'+str('p'*i)+'=pqq(z'+str('p'*i)+'), ')
        cpp.write('pqg_z'+str('p'*i)+'=pqg(z'+str('p'*i)+'), ')
        cpp.write('pgg_z'+str('p'*i)+'=pgg(z'+str('p'*i)+');\n')
    for str_prod in d:
        cpp.write(ind+d[str_prod]+'='+str_prod+';\n')
    i=0
    cpp.write(ind+'std::vector<Parton> state;\n')
    str_if=''
    while True:
        str_weight = 'weight['+str(i)+']'
        for line in open(filename,'r'):
            if line[0] != '#' and line.strip():
                str_prod = ''
                for token in line.split():
                    if token[0] == 'p':
                        str_prod+=str(token)+'*'
                str_prod=str_prod[:-1]
                if d[str_prod]==str_weight:
                    cpp.write(ind+'state.clear();\n')
                    k=0
                    sign=-1
                    for token in line.split():
                        if token[0] == 'G':
                            mom=token[token.find("[")+1:token.find("]")]
                            k+=1
                            cpp.write(ind+'state.push_back(Parton('+str(mom)+',Parton::g));\n')
                        if token[0] == 'Q':
                            mom=token[token.find("[")+1:token.find("]")]
                            k+=1
                            cpp.write(ind+'state.push_back(Parton('+str(mom)+',Parton::q));\n')
                    if ((k-order)%2):
                        sign=1
                    cpp.write(ind+'_events.push_back(Event(state,('+str(sign)+')*'+str_weight+'));\n')
        i+=1
        if i>=n:
            break


if __name__ == '__main__':
    main()
