#include "lo.hh"
#include "constants.hh"
#include <iostream>
#include <iomanip>
#include <fstream>
#include <gsl/gsl_errno.h>
#include <LHAPDF/LHAPDF.h> 

//////////
// Simple program to write out the inclusive quark and gluon jet spectrum
// obtained from the LO code in the file ../output/spectrum-lo-code.dat
//
// Requires a file with pt bins in ../output/parton-spectrum.dat
// 
// The output file can then be read by inclusive_spectrum.f90,
// using it as input spectrum. For example:
// ./inclusive_spectrum -out ../output/lo-code-R01.dat -R 0.1 -in ../output/spectrum-lo-code.dat
/////////

using namespace std;

int main(){

  vector<double> ptbins;
  LO lo(7000.0);
  double ymin=-0.1,ymax=-2.;

  ifstream file_bins;
  file_bins.open("../output/ptbins.dat");

  while(!file_bins.eof()) {
    double pt;
    file_bins >> pt;
    ptbins.push_back(pt);
  }
  ofstream file_spect;
  file_spect.open ("../output/spectrum-lo-code.dat");

  int nbins(ptbins.size());
  double i(0);
  
  for (vector<double>::iterator ptit=ptbins.begin(); ptit!=ptbins.end(); ++ptit) {
    double sigma_q,sigma_g;
    cout << "\r Status: " << setprecision(3) << 100*i/nbins << "%  " << flush;
    sigma_g=lo.dsigma_dpt(*ptit,2,ymin,ymax);
    sigma_q=lo.dsigma_dpt(*ptit,1,ymin,ymax);
    file_spect << setw(15) << *ptit << setw(15) << sigma_g
	       << setw(15) << sigma_q << "\n";
    ++i;
  }
  file_spect.close();

  return 0;
}
